Error

Enum Error 

pub enum Error {
    Id(Error),
    Expression(Error),
    Other(Box<dyn Error + Send>),
}
Module error.

Modules are the fundamental building blocks of the system, as they allow to register functionality with the system as part of the implementation of the Module trait. As such, Module::setup must be allowed to return any kind of error, which is then converted into a module error.

The enum is deliberately not marked as non-exhaustive, because this type is intended to be the sink for all errors that typically occur in modules, and should never be used in external code. As such, we’re allowed to freely add new variants, moving errors out of Error::Other when necessary.

In order to integrate with error types that are not already covered by the existing variants, the IntoError conversion trait is provided. Please note that this trait should only be used sparingly, and could denote a code smell, since Module::setup implementations should rarely be fallible besides identifier and expression parsing errors, especially not involving side effects like file system or network access. Implementations should only fail due to programmer or configuration errors, which should be covered by the existing variants. In case you think an essential variant is missing, please report it on our issue tracker.

Variants



Id(Error)

Identifier error.



Expression(Error)

Expression error.



Other(Box<dyn Error + Send>)

Other error.