IntoError

Trait IntoError 

pub trait IntoError {
    // Required method
    fn into_error(self) -> Error;
}
Conversion into [Error].

Required Methods

fn into_error(self) -> Error

Converts into a module error.

This method is intended to be used within Module::setup, to make it convenient to convert an arbitrary error into a module error.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl<E> IntoError for E
where E: Error + Send + 'static,