pub trait IntoError {
// Required method
fn into_error(self) -> Error;
}Expand description
Conversion into [Error].
Required Methods
Sourcefn into_error(self) -> Error
fn into_error(self) -> Error
Converts into a module error.
This method is intended to be used within Module::setup, to make
it convenient to convert an arbitrary error into a module error.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".